/**************************************************************************
 * 
 * Copyright 2009 VMware, Inc.
 * All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL TUNGSTEN GRAPHICS AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 **************************************************************************/

#ifndef P_REFCNT_H
#define P_REFCNT_H


#include "p_defines.h"


#ifdef __cplusplus
extern "C" {
#endif


struct pipe_reference
{
   unsigned count;
};


static INLINE void
pipe_reference_init(struct pipe_reference *reference, unsigned count)
{
   reference->count = count;
}


/**
 * Set 'ptr' to point to 'reference' and update reference counting.
 * The old thing pointed to, if any, will be unreferenced first.
 * 'reference' may be NULL.
 *
 * XXX: thread safety issues!
 */
static INLINE bool
pipe_reference(struct pipe_reference **ptr, struct pipe_reference *reference)
{
   bool destroy = FALSE;

   /* bump the reference.count first */
   if (reference) {
      assert(reference->count);
      reference->count++;
   }

   if (*ptr) {
      assert((*ptr)->count);
      if (--(*ptr)->count == 0) {
         destroy = TRUE;
      }
   }

   *ptr = reference;

   return destroy;
}

#ifdef __cplusplus
}
#endif

#endif /* P_REFCNT_H */
